/***************************************************************************
 *   Copyright (C) 2005 by Joe Ferris                                      *
 *   jferris@optimistictech.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.             *
 ***************************************************************************/

#include <tqfileinfo.h>
#include <kmimetype.h>
#include <kiconloader.h>
#include <tdeio/previewjob.h>

#include "document.h"

Document::Document(TQString _path, TQString _absPath, bool _showPreview)
		: KatapultItem()
{
	this->_path = _path;
	this->_absPath = _absPath;
	this->_gotpix = false;
	this->_showPreview=_showPreview;
	
	TQFileInfo file(_absPath);
	_name = file.fileName();
}

Document::~Document()
{
}

TQPixmap Document::icon(int size) const
{
	KURL u;
	TDEIO::PreviewJob *doc;
	u.setPath(_absPath);
	if (this->_showPreview)
	{

		doc=TDEIO::filePreview(u,size);
		connect(doc, TQT_SIGNAL(gotPreview(const KFileItem*, const TQPixmap&)),
			TQT_SLOT(gotPreview(const KFileItem*, const TQPixmap&)));
	
		if(this->_gotpix)return this->_pix;
		return KMimeType::pixmapForURL(u,0, TDEIcon::NoGroup, size, TDEIcon::DefaultState, 0L);
	}
	else
	{
		return KMimeType::pixmapForURL(u,0, TDEIcon::NoGroup, size, TDEIcon::DefaultState, 0L);
	}
}

void Document::gotPreview(const KFileItem *item, const TQPixmap& pixmap )
{
	this->_pix = pixmap; this->_gotpix=true; 
	emit itemChanged();
}

TQString Document::text() const
{
	return _path;
}

TQString Document::path() const
{
	return _absPath;
}

TQString Document::name() const
{
	return _name;
}

#include "document.moc"
